@@ -1,4 +1,5 @@
 import warnings
+import logging
 
 import lightkurve as lk
 import numpy as np
@@ -8,7 +9,9 @@ from astropy.coordinates import SkyCoord
 from astropy.stats import mad_std
 from astropy.utils.exceptions import AstropyUserWarning
 from astroquery.exceptions import TableParseError
+from astroquery.gaia import Gaia
 from astroquery.jplhorizons import Horizons
+from astroquery.mast import Catalogs
 from astroquery.simbad import Simbad
 from matplotlib import pyplot as plt
 from matplotlib.offsetbox import AnchoredText
@@ -16,14 +19,28 @@ from matplotlib.ticker import MaxNLocator
 from retrying import retry
 from wotan import flatten
 
-from .utils import CANDIDATES_COLUMNS, extend, fill_gaps, find_consecutive, get_flare_probability
+from .utils import (
+    CANDIDATES_COLUMNS,
+    extend,
+    fill_gaps,
+    find_consecutive,
+    get_flare_probability,
+)
+
+logging.getLogger("astroquery").setLevel(logging.WARNING)
 
 CUSTOM_SIMBAD = Simbad()
 CUSTOM_SIMBAD.add_votable_fields("otype")
 CUSTOM_SIMBAD.add_votable_fields("otypes")
 
+Gaia.MAIN_GAIA_TABLE = "gaiadr3.gaia_source"
+Gaia.ROW_LIMIT = -1
+
 CVs = ["CataclyV*", "CataclyV*_Candidate", "Nova", "Nova_Candidate"]
 
+# Zero point TESS flux (from Sullivan 2017)
+TESS_FLUX0 = 4.03e-6 * u.erg / u.s / u.cm**2
+
 
 def load_from_lightkurve(lc):
     with warnings.catch_warnings():
@@ -110,7 +127,10 @@ class FlareLightCurve(lk.LightCurve):
                 eclipse_mask = np.logical_xor(eclipse_mask, self.flux.mask)
 
             i_start_array, i_stop_array = find_consecutive(
-                np.nonzero(eclipse_mask)[0], 3, gap=1.2 * self.meta["TIMEDEL"], data=self.time.value
+                np.nonzero(eclipse_mask)[0],
+                3,
+                gap=1.2 * self.meta["TIMEDEL"],
+                data=self.time.value,
             )
 
             lc = self.copy()
@@ -121,7 +141,13 @@ class FlareLightCurve(lk.LightCurve):
                         t_stop = self.time.value[i_stop_array[i]]
                         duration = t_stop - t_start
                         t_start_ext, t_stop_ext = extend(
-                            lc.time.value, (lc.flux - 1) / sigma, t_start, t_stop, duration, n_sigma=0, mode=-1
+                            lc.time.value,
+                            (lc.flux - 1) / sigma,
+                            t_start,
+                            t_stop,
+                            duration,
+                            n_sigma=0,
+                            mode=-1,
                         )
                         lc.flux[(self.time.value >= t_start_ext) & (self.time.value <= t_stop_ext)] = np.nan
 
@@ -145,7 +171,12 @@ class FlareLightCurve(lk.LightCurve):
 
             t_max_extend = (max_extend_indexes + 0.2) * self.meta["TIMEDEL"]
             t_start_ext, t_stop_ext = extend(
-                lc.time.value, lc.standardized_flux, t_start, t_stop, t_max_extend, n_right=2
+                lc.time.value,
+                lc.standardized_flux,
+                t_start,
+                t_stop,
+                t_max_extend,
+                n_right=2,
             )
             i_start_ext_array[i] = np.nonzero(self.time.value == t_start_ext)[0][0]
             i_stop_ext_array[i] = np.nonzero(self.time.value == t_stop_ext)[0][0]
@@ -171,7 +202,7 @@ class FlareLightCurve(lk.LightCurve):
 
         self.add_columns([rolling_std, standardized_flux], names=["rolling_std", "standardized_flux"])
 
-    @retry(stop_max_attempt_number=10)
+    @retry(stop_max_attempt_number=5)
     def _query_object_type_from_simbad(self):
         """Query object info from Simbad."""
 
@@ -185,7 +216,9 @@ class FlareLightCurve(lk.LightCurve):
                     query_result = CUSTOM_SIMBAD.query_region(obj_coord, radius=3 * u.arcsec)
                     if len(query_result) > 1:
                         simbad_coords = SkyCoord(
-                            query_result["RA"].data.data, query_result["DEC"].data.data, unit=(u.hourangle, u.deg)
+                            query_result["RA"].data.data,
+                            query_result["DEC"].data.data,
+                            unit=(u.hourangle, u.deg),
                         )
                         distance = obj_coord.separation(simbad_coords)
                         query_result = query_result[distance.value.argmin()]
@@ -199,7 +232,39 @@ class FlareLightCurve(lk.LightCurve):
             if "CV*" in query_result["OTYPES"][0].split("|"):
                 self.meta["otype"] = "CataclyV*"
 
-    @retry(stop_max_attempt_number=10)
+    @retry(stop_max_attempt_number=5)
+    def _calculate_stellar_luminosity(self):
+        # Query parallax from Gaia DR3
+        coord = SkyCoord(self.ra, self.dec, unit="deg", frame="icrs", equinox="J2000")
+        radius = u.Quantity(5, u.arcsec)
+        try:
+            gaia_dr3_data = Gaia.cone_search_async(coord, radius).get_results()[0]
+            plx = gaia_dr3_data["parallax"]
+            if hasattr(plx, "mask"):
+                plx = np.nan
+        except IndexError:
+            plx = np.nan
+
+        # Query TESS mag from TIC
+        tic_data = Catalogs.query_object(self.label, radius=0.02, catalog="TIC")
+        tic_data.add_index("ID")
+        try:
+            target_row = tic_data.loc[str(self.ticid)]
+            t_mag = target_row["Tmag"]
+        except KeyError:
+            t_mag = np.nan
+
+        # Calculate stellar luminosity
+        if np.isnan(plx) or np.isnan(t_mag):
+            self.meta["lum"] = np.nan
+        else:
+            plx /= 1000
+            dist = 1 / plx * u.pc
+            flux = 10 ** (-t_mag / 2.5) * TESS_FLUX0
+            lum = 4 * np.pi * dist.to(u.cm) ** 2 * flux
+            self.meta["lum"] = lum.value
+
+    @retry(stop_max_attempt_number=5)
     def _is_sso(self, i_peak: int, radius: float = 8):
         """Check if a candidate is caused by a Solar System Object (SSO) encounter."""
 
@@ -210,17 +275,30 @@ class FlareLightCurve(lk.LightCurve):
             res = self.query_solar_system_objects(cadence_mask=mask, radius=radius * 21 / 3600, show_progress=False)
         except OSError:
             res = self.query_solar_system_objects(
-                cadence_mask=mask, radius=radius * 21 / 3600, cache=False, show_progress=False
+                cadence_mask=mask,
+                radius=radius * 21 / 3600,
+                cache=False,
+                show_progress=False,
             )
 
         if res is not None:
             ap_mag = np.zeros(len(res))
             for row in res.itertuples():
                 try:
-                    obj = Horizons(id=row.Name.strip(), location="500@-95", epochs=row.epoch, id_type="smallbody")
+                    obj = Horizons(
+                        id=row.Name.strip(),
+                        location="500@-95",
+                        epochs=row.epoch,
+                        id_type="smallbody",
+                    )
                     eph = obj.ephemerides(quantities=9)
                 except ValueError:
-                    obj = Horizons(id=row.Num, location="500@-95", epochs=row.epoch, id_type="smallbody")
+                    obj = Horizons(
+                        id=row.Num,
+                        location="500@-95",
+                        epochs=row.epoch,
+                        id_type="smallbody",
+                    )
                     eph = obj.ephemerides(quantities=9)
                 try:
                     ap_mag[row.Index] = eph["V"].value
@@ -259,7 +337,8 @@ class FlareLightCurve(lk.LightCurve):
         )
         trend_flux += model_flux
         self.add_columns(
-            [detrended_flux * self.flux.unit, trend_flux * self.flux.unit], names=["detrended_flux", "trend_flux"]
+            [detrended_flux * self.flux.unit, trend_flux * self.flux.unit],
+            names=["detrended_flux", "trend_flux"],
         )
         self._standardize()
 
@@ -270,7 +349,10 @@ class FlareLightCurve(lk.LightCurve):
             i_outliers = np.nonzero(self.standardized_flux > n_sigma)[0]
 
             i_start_array, i_stop_array = find_consecutive(
-                i_outliers, n_consecutive, gap=1.2 * self.meta["TIMEDEL"], data=self.time.value
+                i_outliers,
+                n_consecutive,
+                gap=1.2 * self.meta["TIMEDEL"],
+                data=self.time.value,
             )
 
             if i_start_array is not None:
@@ -332,6 +414,7 @@ class FlareLightCurve(lk.LightCurve):
                 amp = np.zeros(len(self.candidates))
                 dur = np.zeros(len(self.candidates))
                 ed = np.zeros(len(self.candidates))
+                self._calculate_stellar_luminosity()
 
                 n_row = 0
                 for row in self.candidates.itertuples():
@@ -346,6 +429,11 @@ class FlareLightCurve(lk.LightCurve):
                 self.candidates.amp = np.round(amp, 3)
                 self.candidates.dur = np.round(dur, 1)
                 self.candidates.ed = np.round(ed, 2)
+                if np.isnan(self.lum):
+                    self.candidates.energy = np.full(len(self.candidates), np.nan)
+                else:
+                    energy = ed * self.lum
+                    self.candidates.energy = [np.format_float_scientific(x, precision=2) for x in energy]
 
     def find_flares(self):
         self.detrend()
@@ -363,7 +451,10 @@ class FlareLightCurve(lk.LightCurve):
             output.t_peak = np.round(self.candidates.t_peak, 7)
             output.t_stop = np.round(self.candidates.t_stop, 7)
 
-            output.to_csv(folder / "{}-S{}.csv".format(self.label.replace(" ", ""), self.sector), index=False)
+            output.to_csv(
+                folder / "{}-S{}.csv".format(self.label.replace(" ", ""), self.sector),
+                index=False,
+            )
 
     def plot_candidates(self, figure_folder, threshold=0.5):
         """Plot the candidates."""
@@ -373,7 +464,10 @@ class FlareLightCurve(lk.LightCurve):
             finite_mask = np.isfinite(self.standardized_flux)
 
             self.candidates.loc[self.candidates.sso, "flare_prob"] = -999
-            cond_list = [self.candidates.flare_prob >= threshold, self.candidates.flare_prob == -999]
+            cond_list = [
+                self.candidates.flare_prob >= threshold,
+                self.candidates.flare_prob == -999,
+            ]
             candidate_type_array = np.select(cond_list, ["flare", "sso"], "non-flare")
             color_array = np.select(cond_list, ["tab:red", "tab:orange"], "tab:gray")
             with plt.style.context(lk.MPLSTYLE):
@@ -393,7 +487,12 @@ class FlareLightCurve(lk.LightCurve):
 
                     ax_original_lc = fig.add_subplot(221)
                     self.scatter(ax=ax_original_lc, label="")
-                    self.plot(ax=ax_original_lc, column="trend_flux", color="tab:red", label="")
+                    self.plot(
+                        ax=ax_original_lc,
+                        column="trend_flux",
+                        color="tab:red",
+                        label="",
+                    )
                     ax_original_lc.set_xlabel("")
                     ax_original_lc.set_ylabel("")
                     ax_original_lc.set_xticklabels([])
@@ -440,10 +539,16 @@ class FlareLightCurve(lk.LightCurve):
                             f"flare prob: {row.flare_prob}\n"
                             f"snr: {row.snr}\n"
                             f"amp: {row.amp}\n"
-                            f"dur: {row.dur}\n m"
-                            f"ed: {row.ed} s"
+                            f"dur: {row.dur} m\n"
+                            f"ed: {row.ed} s\n"
+                            f"{row.energy} erg"
                         )
-                    at = AnchoredText(box_str, loc="upper right", frameon=True, prop={"multialignment": "right"})
+                    at = AnchoredText(
+                        box_str,
+                        loc="upper right",
+                        frameon=True,
+                        prop={"multialignment": "right"},
+                    )
                     at.patch.set_boxstyle("round,pad=0.,rounding_size=0.2")
                     ax_event.add_artist(at)
 
@@ -455,15 +560,29 @@ class FlareLightCurve(lk.LightCurve):
                     i_event = np.zeros_like(self.time.value, dtype=bool)
                     i_event[row.i_start : row.i_stop + 1] = True
                     i_event = i_event & finite_mask
-                    self[i_plot].plot(ax=ax_event, column="standardized_flux", label="", lw=1, ms=5, marker=".")
+                    self[i_plot].plot(
+                        ax=ax_event,
+                        column="standardized_flux",
+                        label="",
+                        lw=1,
+                        ms=5,
+                        marker=".",
+                    )
                     self[i_event].plot(
-                        ax=ax_event, column="standardized_flux", color=color, label="", lw=1.5, ms=5, marker="."
+                        ax=ax_event,
+                        column="standardized_flux",
+                        color=color,
+                        label="",
+                        lw=1.5,
+                        ms=5,
+                        marker=".",
                     )
 
                     ax_event.xaxis.set_major_locator(MaxNLocator(5))
                     ax_event.yaxis.set_major_locator(MaxNLocator(integer=True))
                     ax_event.set_xlim(
-                        row.t_start - t_extend + self.meta["TIMEDEL"], row.t_stop + t_extend - self.meta["TIMEDEL"]
+                        row.t_start - t_extend + self.meta["TIMEDEL"],
+                        row.t_stop + t_extend - self.meta["TIMEDEL"],
                     )
                     if self.standardized_flux[i_plot].min() < -8:
                         ax_event.set_ylim(bottom=-8)
@@ -474,7 +593,14 @@ class FlareLightCurve(lk.LightCurve):
                     ax_event.yaxis.tick_right()
 
                     ax_label.set_yticks(ax_detrended_lc.get_yticks())
-                    ax_label.tick_params(colors="w", which="both", top=False, bottom=False, left=False, right=False)
+                    ax_label.tick_params(
+                        colors="w",
+                        which="both",
+                        top=False,
+                        bottom=False,
+                        left=False,
+                        right=False,
+                    )
 
                     period = "{:.2f}d".format(self.period) if self.period > 0 else "/"
                     title = f"{self.label}, Sector {self.sector}, {self.otype}, P={period}"
